// Blynk Authentication
#define BLYNK_TEMPLATE_ID "TMPL6tFiJq65y"
#define BLYNK_TEMPLATE_NAME "Demo"
#define BLYNK_AUTH_TOKEN "sij-WfSSWsBmjiEjEcQi2nr4s7DdH_Hb"
#define BLYNK_PRINT Serial

#include <WiFi.h>
#include <WiFiClient.h>
#include <BlynkSimpleEsp32.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "SPIFFS.h"
#include "AudioFileSourceSPIFFS.h"
#include "AudioGeneratorMP3.h"
#include "AudioOutputI2S.h"
#include <Adafruit_NeoPixel.h>



#define LED_PIN    18  // Define the pin where the NeoPixel strip is connected
#define NUM_LEDS   12  // Define the number of LEDs in the strip

Adafruit_NeoPixel strip(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);  // Initialize the NeoPixel strip



void playTrack(int trackNumber);
void stopPlayback();
long measureDistance();
void setupUltrasonic();
void sendToMake1();
void getRemainingTimeAndAnnounce();

// Wi-Fi credentials
char ssid[] = "RUNI-Dorms";
char pass[] = ""; // Update with Wi-Fi password if needed


// Make.com Webhook URL
String makeWebhookUrl = "https://hook.eu2.make.com/4n30nfwqauwmah7rh9cp2w9dvkeqdfka";  // Replace with your Make.com Webhook URL

// Audio setup variables
AudioGeneratorMP3 *mp3;
AudioFileSourceSPIFFS *file;
AudioOutputI2S *out;

#define TRIGGER_PIN  16
#define ECHO_PIN     17
long duration;
int distance;
unsigned long lastSensorCheck = 0;
const int sensorInterval = 200; // ms
bool isPlaying = false;

void setup() {
  Serial.begin(115200);

  WiFi.begin(ssid, pass);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nWi-Fi connected!");

  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, pass);
  Serial.println("Connected to Blynk");

  if (!SPIFFS.begin(true)) {
    Serial.println("SPIFFS initialization failed!");
    return;
  }
  Serial.println("SPIFFS mounted successfully.");

  strip.begin();  // Initialize the NeoPixel strip
  strip.show();   // Initialize all LEDs to off

  out = new AudioOutputI2S();
  out->SetPinout(26, 25, 22);
  out->SetGain(0.8);

  setupUltrasonic();
}

void loop() {
  Blynk.run();

  if (mp3 != nullptr && mp3->isRunning()) {
    if (!mp3->loop()) {
      Serial.println("Track finished");
      stopPlayback();
      isPlaying = false;
      Serial.println("Ready for next trigger");
    }
  }

  unsigned long now = millis();
  if (now - lastSensorCheck > sensorInterval) {
    lastSensorCheck = now;
    long currentDistance = measureDistance();

    if (currentDistance < 20 && !isPlaying) {
      Serial.println("Object detected - Playing music");
      playTrack(1);
      isPlaying = true;
    }

  }
}

void setupUltrasonic() {
  pinMode(TRIGGER_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
}

long measureDistance() {
  digitalWrite(TRIGGER_PIN, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIGGER_PIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGGER_PIN, LOW);
  duration = pulseIn(ECHO_PIN, HIGH);
  distance = duration * 0.034 / 2;
  return distance;
}

BLYNK_WRITE(V1) {
  int buttonState = param.asInt();
  if (buttonState == 1) {
    Serial.println("V1 Button Pressed! Sending Webhook...");
    sendToMake1();
  }
}

BLYNK_WRITE(V2) {
  int buttonState = param.asInt();
  if (buttonState == 1) {
    Serial.println("V2 Button Pressed! Getting remaining time...");
    getRemainingTimeAndAnnounce();
  }
}

int buttonPressCount = 0;

void sendToMake1() {
  if (WiFi.status() == WL_CONNECTED) {
    HTTPClient http;
    http.begin(makeWebhookUrl);
    http.addHeader("Content-Type", "application/json");

    String jsonPayload = "{\"event\":\"button_pressed\",\"device\":\"ESP32\"}";
    int httpResponseCode = http.POST(jsonPayload);

    if (httpResponseCode > 0) {
      Serial.println("Make.com Webhook Sent Successfully!");
    } else {
      Serial.println("Error Sending Webhook: " + String(httpResponseCode));
    }

    http.end();
  } else {
    Serial.println("Wi-Fi Disconnected, Cannot Send Webhook!");
  }
   buttonPressCount++;
   Blynk.virtualWrite(V3,buttonPressCount);
   stopPlayback();
   isPlaying = false;
   playTrack(4);
   lightUpGreen();
}

void getRemainingTimeAndAnnounce() {
  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("Wi-Fi Disconnected, Cannot Get Remaining Time!");
    return;
  }

  HTTPClient http;
  http.begin(makeWebhookUrl);
  http.addHeader("Content-Type", "application/json");

  String jsonPayload = "{\"request\":\"get_remaining_time\"}";
  int httpResponseCode = http.POST(jsonPayload);

  if (httpResponseCode > 0) {
    String response = http.getString();
    Serial.println("Response: " + response);

    DynamicJsonDocument doc(512);
    DeserializationError error = deserializeJson(doc, response);

    if (error) {
      Serial.print("JSON parsing failed: ");
      Serial.println(error.c_str());
      http.end();
      return;
    }

    int remainingMinutes = doc["remaining_minutes"] | -1;
    Serial.print("Remaining Minutes: ");
    Serial.println(remainingMinutes);


    Blynk.virtualWrite(V4, String(remainingMinutes) + " minutes");
    
    // Force stop any current playback and reset the flag
    stopPlayback();
    isPlaying = false;
    
    
    if (remainingMinutes > 30) {
      playTrack(2);
      lightUpBlue();  // Turn LEDs blue for more than 30 minutes left
    } else if (remainingMinutes >= 0) {
      playTrack(3);
      lightUpRed();   // Turn LEDs red for less than 30 minutes left
    }
  } else {
    Serial.println("HTTP Error: " + String(httpResponseCode));
  }

  http.end();
}

void playTrack(int trackNumber) {
  stopPlayback();  // Ensure everything is cleaned up first

  String trackName = "/track_" + String(trackNumber) + ".mp3";

  if (!SPIFFS.exists(trackName)) {
    Serial.println("Error: File " + trackName + " doesn't exist!");
    return;
  }

  file = new AudioFileSourceSPIFFS(trackName.c_str());
  
  mp3 = new AudioGeneratorMP3();

  if (!mp3->begin(file, out)) {
    Serial.println("mp3->begin failed");
    stopPlayback();
    return;
  }

  Serial.println("Started playing " + trackName);
  isPlaying=true;
}


void stopPlayback() {
  if (mp3 != nullptr) {
    if (mp3->isRunning()) {
      mp3->stop();
    }
    delete mp3;
    mp3 = nullptr;
  }
  if (file != nullptr) {
    delete file;
    file = nullptr;
  }
  // Note: We don't delete the out object here, we keep it for reuse
  isPlaying = false;
}


void lightUpRed() {
  unsigned long startMillis = millis();
  unsigned long duration = 6000;  // Flash duration of 6 seconds
  unsigned long flashDelay = 500; // Flash interval (500ms)
  unsigned long flashStartTime = millis() + 3000;  // Flash will start after 3 seconds

  while (millis() - startMillis < duration) {
    if (millis() >= flashStartTime) {
      // Turn LEDs red
      for (int i = 0; i < NUM_LEDS; i++) {
        strip.setPixelColor(i, strip.Color(255, 0, 0));  // Red color
      }
      strip.show();
      delay(flashDelay);  // Wait for 500ms
    
      // Turn LEDs off
      for (int i = 0; i < NUM_LEDS; i++) {
        strip.setPixelColor(i, strip.Color(0, 0, 0));  // Off
      }
      strip.show();
      delay(flashDelay);  // Wait for 500ms
    }
  }

  // Make sure LEDs are turned off after flashing
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(0, 0, 0));  // Turn off LEDs
  }
  strip.show();
}


void lightUpBlue() {
  int numLeds = NUM_LEDS; // Assuming you have 12 LEDs
  int delayTime = 100; // Speed of the moving effect (in milliseconds)
  int numCycles = 2; // Number of cycles (animations)

  // Run the blue animation twice
  for (int cycle = 0; cycle < numCycles; cycle++) {
    // Moving left to right
    for (int i = 0; i < numLeds; i++) {
      strip.setPixelColor(i, strip.Color(0, 0, 255));  // Blue color
      strip.show();
      delay(delayTime);

      // Turn off the previous LED
      if (i > 0) {
        strip.setPixelColor(i - 1, strip.Color(0, 0, 0));  // Turn off the previous LED
      }
    }

    // Moving right to left
    for (int i = numLeds - 1; i >= 0; i--) {
      strip.setPixelColor(i, strip.Color(0, 0, 255));  // Blue color
      strip.show();
      delay(delayTime);

      // Turn off the next LED
      if (i < numLeds - 1) {
        strip.setPixelColor(i + 1, strip.Color(0, 0, 0));  // Turn off the next LED
      }
    }
  }

  // Turn off LEDs after animation
  for (int i = 0; i < numLeds; i++) {
    strip.setPixelColor(i, strip.Color(0, 0, 0));  // Turn off LEDs
  }
  strip.show();
}

void lightUpGreen() {
  // Light up NeoPixel strip green
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(0, 255, 0));  // Green color
  }
  strip.show();

  delay(3000);  // Keep green light on for 3 seconds

  // Turn off NeoPixel after 3 seconds
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(0, 0, 0));  // Turn off
  }
  strip.show();
}
